module Feathr
  module Api

    class FeathrEndpoint
      attr_accessor :client, :prepended_path

      def initialize(client: Feathr::Client.default, prepended_path: '')
        @client         = client
        @prepended_path = prepended_path
      end

      class << self
        def api_path(path)
          define_method(:api_path) do
            [
              prepended_path,
              path.gsub(/\/?$/, '/')
            ].join('/').gsub(/^\//,'')
          end
        end

        def feathr_object(klass)
          define_method(:feathr_object) do
            klass
          end
        end

        def feathr_endpoints(*endpoints)
          endpoints.each do |name|
            self.send("define_#{ name }")
          end
        end

        def define_all
          define_method(:all) do
            client.request(method: :get, path: api_path) do |data|
              data.map do |resource|
                if feathr_object == Feathr::Api::Membership
                  feathr_object.new(resource)
                else
                  id = resource["url"][/\/(\d*)\/$/, 1]
                  feathr_object.new(resource, id)
                end
              end
            end
          end

          define_method(:first) do |count=1|
            count > 1 ? all.first(count) : all.first
          end

          define_method(:last) do |count=1|
            count > 1 ? all.last(count) : all.last
          end
        end

        def define_find
          define_method(:find) do |id|
            path = "#{ api_path }#{ id }/"
            client.request(method: :get, path: path) do |data|
              if feathr_object == Feathr::Api::Membership
                feathr_object.new(data)
              else
                feathr_object.new(data, id)
              end
            end
          end
        end

        def define_create
          define_method(:create) do |query|
            client.request(method: :post, path: api_path, query: query) do |data|
              if feathr_object == Feathr::Api::Membership
                feathr_object.new(data)
              else
                id = data["url"][/\/(\d*)\/$/, 1]
                feathr_object.new(data, id)
              end
            end
          end
        end

        def define_update
          define_method(:update) do |id, query|
            path = "#{ api_path }#{ id }/"
            client.request(method: :patch, path: path, query: query) do |data|
              if feathr_object == Feathr::Api::Membership
                feathr_object.new(data)
              else
                id = data["url"][/\/(\d*)\/$/, 1]
                feathr_object.new(data, id)
              end

            end
          end
        end

        def define_destroy
          define_method(:destroy) do |id|
            path = "#{ api_path }#{ id }/"
            client.request(method: :delete, path: path) do |data|
              data
            end
          end
        end
      end
    end

  end
end
